// CustomtextColumn
// By wowbagger

// -------------------- Options
var gbUseLogging=true;

//Default values for extendedConfig
var defaultCustomColumns = {
	columns: [{
		name:"RegExp1", 
		displayName:"RegExp1",

		//Column settings
		type: "string",
		enableAutoGroup: false,//flag (true/false) as described in the DOpus documentation
		enableSort: false,//flag (true/false) as described in the DOpus documentation
		justify: "left", //"left", "right", "center" or "path". The default is "left".
		defwidth: 20,
		datetimeformat: "yyyy-mm-dd", //Date Format used to parse the extracted date information (time not realy supported at the moment!)
		autorefresh: false,
		regexp: "(.*)", // Will select everyting
		captureGroup: 0,// 
		firstValid: false,  //(true/false) 
		inputItemProperty: "name_stem", // name_stem, realpath, name, ext, contents
		subjectPrefix: "", 
	 	filter: "all" // all, files, folders
	}]};

// Functions
var doVars = DOpus.vars;
var doLogCmd = DOpus.NewCommand;
var configVarKey = "RegexpCustomColumnsConfig";

function OnInit(initData) {
  // Provide basic information about the script by initializing the properties of the ScriptInitData object
  initData.name = "Rebexp Columns";
  initData.desc = "Adds columns and displays values matched by regexp against filename.";
  initData.copyright = "2014 wowbagger";
  initData.min_version = "12.0"
  initData.version = "1.4";
  initData.default_enable = true;
}

function OnAddCommands(addCmdData)
{
	var cmd = addCmdData.AddCommand();
	cmd.name = "RegExpColumnsConfigure";
	cmd.method = "OnRegExpColumnsConfigure";
	cmd.desc = "Configure the RegExp columns";
	cmd.label = "RegExpColumnsConfigure";
}

function OnAddColumns(addColData)
{
	//LogMessage("OnAddColumns called", true)
	var customColumns = LoadCustomColumns();
	if(customColumns && customColumns.columns.length > 0)
	{
		//var customColumns = JSON.parse(Script.config.CustomColumns);
		for(i=0;i<customColumns.columns.length;i++) {
			if(addColData,customColumns.columns[i])
			{
				AddColumn(addColData, //iAddColumn
					customColumns.columns[i].name, //name
					customColumns.columns[i].displayName, //label
					customColumns.columns[i].type, //type
					customColumns.columns[i].defwidth, //defwidth
					customColumns.columns[i].enableAutoGroup, //autogroup
					customColumns.columns[i].autorefresh, //autorefresh
					customColumns.columns[i].justify, //justify
					true //multicol
				);
			}
		}
	}
}

// The OnScriptConfigChange event can be implemented by a script add-in to receive 
// notification whenever the user modifies the script's configuration via the Preferences editor.
function OnScriptConfigChange(configChangeData)
{
	LogMessage("OnScriptConfigChange " + (configChangeData ? (configChangeData.changed ? configChangeData.changed.length : 'nothing changed') :'null' , true));
	for (var i = 0; i < configChangeData.changed.length; i++) {
		LogMessage("OnScriptConfigChange " + i + ":" + configChangeData.changed[i], true)
	}

	Script.InitColumns;
}

// Implement the ExternalCompare command
function OnRegExpColumnsConfigure(scriptCmdData)
{
    var dlg = DOpus.Dlg;
    
    dlg.window = scriptCmdData.func.sourcetab;
    dlg.template = "RegExpColumnsConfigure";
    dlg.detach = true;
    dlg.Show();
    onDialogPopulateColumnsList(dlg);

    var closeDialog;
    while (!closeDialog) {
		var msg = dlg.GetMsg();
		if (!msg.result) break;
		//LogMessage("Msg: " + msg.control + " value: " + msg.value + " event:" + msg.event);
		switch (msg.control) {
			case "columnsList" :
				onDialogConfigSelected(dlg, msg);
				break;
			case "buttonAdd" :
				onDialogAddColumn(dlg, msg);
				break;
			case "buttonDelete" :
				onDialogDeleteColumn(dlg, msg);
				break;
			case "buttonApply" :
				 onDialogApplyChanges(dlg, msg);
				break;
			case "buttonClipboard" :
				 CopyToClipboard();
				break;
			case "buttonClearSettings" :
				ClearCustomColumns();
				closeDialog = true;
				break;
			case "columnType" :
				onDialogColumnType(dlg, msg);
				break;
			case "buttonOK" :
				//params = DialogClosed(dlg, msg);
				closeDialog = true;
				break;
		} 
    }
}

function onDialogPopulateColumnsList(dlg)
{
    var columnsList = dlg.Control("columnsList");
    var customColumnsConfig = LoadCustomColumns();//JSON.parse(Script.config.CustomColumns);

	var selectedColumn;
    for(var i = 0; i < customColumnsConfig.columns.length; ++i) 
    {
    	if(customColumnsConfig.columns[i])
    	{
			var location = columnsList.AddItem(customColumnsConfig.columns[i].name);
			var item = columnsList.GetItemAt(location);
			if(!selectedColumn)
			{ 
				item.selected = true;
				selectedColumn = customColumnsConfig.columns[i];
				onDialogConfigSelected(dlg);
			}
 			LogMessage("item: " + item.index + " location: " + location + " product:" + customColumnsConfig.columns[i].name);
 		}
    }
}

// Dialog functions
function onDialogColumnType(dlg)
{
	var selectedType = GetSelectedItem(dlg ,"columnType");
	LogMessage("onDialogColumnType: " + ((selectedType && selectedType.name) ? selectedType.name : 'null'));
	dlg.Control("columnDateTimeFormat").enabled = (selectedType && selectedType.name == "datetime");
}

function onDialogConfigSelected(dlg)
{
	var selectedUiColumn = GetSelectedItem(dlg ,"columnsList");
	//LogMessage("onDialogConfigSelected: " + ((selectedUiColumn && selectedUiColumn.name) ? selectedUiColumn.name : 'null'));
	if(!selectedUiColumn) return;

	var selectedColumnConfig = GetRegexpColumnConfig(selectedUiColumn.name);

	dlg.Control("columnName").value = selectedColumnConfig ? selectedColumnConfig.name : "";
	dlg.Control("columnDisplayName").value = selectedColumnConfig ? selectedColumnConfig.displayName : "";
	dlg.Control("columnRegExp").value = selectedColumnConfig ? selectedColumnConfig.regexp : "";
	SetSelectedItem(dlg, "columnType", (selectedColumnConfig ? selectedColumnConfig.type : ""));
	dlg.Control("columnDateTimeFormat").value = selectedColumnConfig ? selectedColumnConfig.datetimeformat : "";
	dlg.Control("columnDateTimeFormat").enabled = (selectedColumnConfig && selectedColumnConfig.type == "datetime");
	dlg.Control("columnDefaultWidth").value = selectedColumnConfig ? selectedColumnConfig.defwidth : "";
	dlg.Control("columnEnableGrouping").value = selectedColumnConfig ? selectedColumnConfig.nogroup : "";
	dlg.Control("columnEnableSort").value = selectedColumnConfig ? selectedColumnConfig.nosort : "";
	SetSelectedItem(dlg, "columnJustify", (selectedColumnConfig ? selectedColumnConfig.justify : ""));
	dlg.Control("columnCaptureGroup").value = selectedColumnConfig ? selectedColumnConfig.captureGroup : "";
	dlg.Control("columnFirstValid").value = selectedColumnConfig ? selectedColumnConfig.firstValid : "";
	SetSelectedItem(dlg, "columnInput", (selectedColumnConfig ? selectedColumnConfig.inputItemProperty : ""));
	SetSelectedItem(dlg, "columnFilter", (selectedColumnConfig ? selectedColumnConfig.filter : ""));
}

function onDialogApplyChanges(dlg, msg)
{
	var selectedUiColumn = GetSelectedItem(dlg ,"columnsList");
	//LogMessage("onDialogApplyChanges: " + selectedUiColumn ? selectedUiColumn.name : 'null');
	if(!selectedUiColumn) return;
	
	var oldName = selectedUiColumn.name;

	var customColumns = LoadCustomColumns();
	for(var i = 0; i < customColumns.columns.length; ++i) {
		if(customColumns.columns[i] && customColumns.columns[i].name.toLowerCase() == oldName.toLowerCase()) {
			
			var selectedColumn = customColumns.columns[i];
			selectedColumn.name = dlg.Control("columnName").value;
			selectedColumn.displayName = dlg.Control("columnDisplayName").value;
			selectedColumn.regexp = dlg.Control("columnRegExp").value;
			var selectedType = GetSelectedItem(dlg, "columnType");
			selectedColumn.type = selectedType ? selectedType.name : "";
			selectedColumn.defwidth = dlg.Control("columnDefaultWidth").value;
			selectedColumn.nogroup = dlg.Control("columnEnableGrouping").value;
			selectedColumn.nosort = dlg.Control("columnEnableSort").value;
			var selectedJustify = GetSelectedItem(dlg, "columnJustify");
			selectedColumn.justify = selectedJustify ? selectedJustify.name : "";
			selectedColumn.datetimeformat = dlg.Control("columnDateTimeFormat").value;
			selectedColumn.captureGroup = dlg.Control("columnCaptureGroup").value;
			selectedColumn.firstValid = dlg.Control("columnFirstValid").value;
			var selectedInput = GetSelectedItem(dlg, "columnInput");
			selectedColumn.inputItemProperty = selectedInput ? selectedInput.name : "";
			var selectedFilter = GetSelectedItem(dlg, "columnFilter");
			selectedColumn.filter = selectedFilter ? selectedFilter.name : "";
		
			SaveCustomColumns(customColumns);
	
			//Script.RefreshColumn(oldName.name);
			selectedUiColumn.name = dlg.Control("columnName").value; 
			Script.RefreshColumn(dlg.Control("columnName").value);

		  	break;
		}
	}
}

function onDialogAddColumn(dlg, msg)
{
  	//LogMessage("onDialogAddColumn");
  
  	var newColumn = defaultCustomColumns.columns[0];
  	var customColumnsConfig = LoadCustomColumns();
	customColumnsConfig.columns.push(newColumn);
	SaveCustomColumns(customColumnsConfig);

	//customColumnsConfig.columns[customColumnsConfig.columns.length - 1]
	AddToColumnsList(dlg, newColumn.name, true);
	onDialogConfigSelected(dlg, msg);
}

function onDialogDeleteColumn(dlg, msg)
{
	var selectedUiColumn = GetSelectedItem(dlg ,"columnsList");
	//LogMessage("onDialogApplyChanges: " + selectedUiColumn ? selectedUiColumn.name : 'null');
	if(!selectedUiColumn) return;

	var customColumns = LoadCustomColumns();
	for(var i = 0; i < customColumns.columns.length; ++i) {
		if(customColumns.columns[i].name.toLowerCase() == selectedUiColumn.name.toLowerCase()) {
			
			delete customColumns.columns[i];

			SaveCustomColumns(customColumns);
			Script.RefreshColumn(selectedUiColumn.name);
		  	break;
		}
	}

	RemoveFromColumnsList(dlg, selectedUiColumn);
	onDialogConfigSelected(dlg, msg);
}

function RemoveFromColumnsList(dlg, column)
{
	var columnsList = dlg.Control("columnsList");
	var index = column.index;
	columnsList.RemoveItem(column);

	if(columnsList.count > 0)
	{
		var selectedColumn = columnsList.GetItemAt((index > 0) ? index -1 : 0);
    	selectedColumn.selected = true;
	}
}

function AddToColumnsList(dlg, name, setSelected)
{
	var columnsList = dlg.Control("columnsList");
 	var location = columnsList.AddItem(name);
 	if(setSelected)
 	{
		var item = columnsList.GetItemAt(location);
		item.selected = true;
	}
}

function GetSelectedItem(dlg, controlName)
{
	//LogMessage("GetSelectedItem: " + controlName);

	var listItems = dlg.Control(controlName);
    for(var i = 0; i < listItems.count; ++i) {
    	var column = listItems.GetItemAt(i);
	
		//LogMessage("GetSelectedItem: " + controlName + " " + i + " of " + listItems.count + " is" + (column.selected ? "selected" : "not selected"));
    	if(column && column.selected)
    	{
			//LogMessage("GetSelectedItem: found " + column.name);
    		return column;
 		}
    }
}

function SetSelectedItem(dlg, controlName, value)
{
	var listItems = dlg.Control(controlName);

	if(typeof(value) == "undefined") 
	{
		var item = listItems.GetItemAt(0);
		//LogMessage("SetSelectedItem: " + controlName + ":defaulted to " + item.name);
		item.selected = true;
		return;
	}

    for(var i = 0; i < listItems.count; ++i) {
    	var item = listItems.GetItemAt(i);
	
		//LogMessage("SetSelectedItem: " + controlName + " " + i + " of " + listItems.count + " is " + item.name  + " looking for " + value);
    	if(item && item.name == value)
    	{
			//LogMessage("SetSelectedItem: " + controlName + ":found " + item.name);
			item.selected = true;
 		}
    }
}

function CopyToClipboard(){

	var customColumns = LoadCustomColumns();
	var config = JSON.stringify(customColumns, undefined, 2).replace(/(\n)+/g,"\r\n");
	var doCmd = DOpus.NewCommand;

	var cmd = "Clipboard SET " + config;
	doCmd.RunCommand(cmd);
}

function SaveCustomColumns(customColumns){

	var newList = new Array();
	for(var i = 0; i < customColumns.columns.length; ++i) {
		if(customColumns.columns[i]) {
			newList.push(customColumns.columns[i]);
		}
	}

	customColumns.columns = newList;

	var config = JSON.stringify(customColumns, undefined, 2).replace(/(\n)+/g,"\r\n");

	LogMessage("SaveCustomColumns: " + config);
	doVars(configVarKey) = config;
    doVars(configVarKey).persist = true;

    Script.InitColumns;
}

function ClearCustomColumns(){
	if(doVars.Exists(configVarKey))
	{
        doVars(configVarKey).Delete()
	}
}

function LoadCustomColumns(){
	//LogMessage("LoadCustomColumnsConfiguration from var");
	//return JSON.parse(Script.config.CustomColumns);
  	var config = doVars.Exists(configVarKey) ? JSON.parse(doVars.Get(configVarKey)) : null;
  	if(!config) 
  	{
  		config = defaultCustomColumns;
  	}

	return config
}

function GetRegexpColumnConfig(name)
{
	var customColumns = LoadCustomColumns();
	for(var i = 0; i < customColumns.columns.length; ++i) {
		if(customColumns.columns[i].name.toLowerCase() == name.toLowerCase()) {
			return customColumns.columns[i];
		}
	}
}

function AddColumn(iAddColumn,name,label,type,defwidth,autogroup,autorefresh, justify, multicol){
	LogMessage("AddColumn: name=" + name + " label=" + label + " type=" + type + " defwidth=" + defwidth + 
						"autogroup=" + autogroup + " autorefresh=" + autorefresh + " justify=" + justify + " multicol=" + multicol);
	var col;
	col=iAddColumn.AddColumn();
	col.name = name;
	col.label = label;
	col.type = type;
	col.method = "OnCustomText";
	col.autogroup = autogroup;///true;
	col.autorefresh = autorefresh;///true;
	col.justify = justify;///"left";
	col.multicol = multicol;///true;
	if(defwidth != null && defwidth != 0)
	{
		col.defwidth=defwidth;
	}  
}


function IsDebugEnabled()
{
  
  if(Script)
  {
	return gbUseLogging || Script.config.DEBUG || doLogCmd.IsSet(Script.config.DebugEnableVar);
  }

  return gbUseLogging;
}

function LogMessage(message, force)
{
  if (force || IsDebugEnabled()) { DOpus.OutputString(message)};
}

// Implement the OnCustomText column (this entry point is an OnScriptColumn event).
function OnCustomText(scriptColData) {
	LogMessage("OnCustomText: column=" + scriptColData.col + " columnsCount=" + scriptColData.columns.length + " itempPath=" + scriptColData.Item.path);
  	// get column's config

	var customColumnsConfig = LoadCustomColumns();
	var columnsConfig = customColumnsConfig.columns;
		
	if (!InArray(scriptColData.col, columnsConfig)) return;
	LogMessage("1: column=" + scriptColData.col + " columnsCount=" + scriptColData.columns.length + " itempPath=" + scriptColData.Item.path);
  
	var results = RunRegExpOnItem(scriptColData.Item, columnsConfig);
	if (results){
		for ( i = 0; i < columnsConfig.length; i++) 
		{
			if (typeof(columnsConfig[i].name) != "undefined")
			{
				var result = results[columnsConfig[i].name];
				if (scriptColData.columns.exists(columnsConfig[i].name))
				{
					switch (columnsConfig[i].type) {
						case "datetime" :
							LogMessage("Datetime for " + columnsConfig[i].name + "[" + columnsConfig[i].type + "]:" + result);
							var parsedDate = parseDate(results[columnsConfig[i].name], columnsConfig[i].datetimeformat);
							scriptColData.columns(columnsConfig[i].name).value = parsedDate ? parsedDate.getVarDate() : null;
							break;
						case "graph" :
						case "igraph" :
						case "percent" :
						case "stars" :
							LogMessage("number for " + columnsConfig[i].name + "[" + columnsConfig[i].type +"]:" + result);
	    					scriptColData.columns(columnsConfig[i].name).value = (isNaN(result) ? 0 : result);
							break;
						default :
							LogMessage("results for " + columnsConfig[i].name + "[" + columnsConfig[i].type +"]:" + result);
    						scriptColData.columns(columnsConfig[i].name).value = result;
				
							break;
					} 
				}
				else
				{
					LogMessage("column not loaded " +columnsConfig[i].name + "["+columnsConfig[i].type +"]:" + result);
				}
			}
		}
	}
}

function RunRegExpOnItem(item, columnConfigurations)
{
	var result = {};
	if(!item) return;

	for ( i = 0; i < columnConfigurations.length; i++) 
	{
		var currentCol = columnConfigurations[i];

  		var fileSubject;
		var fileContents;
		var subject;
		var regexMatch;
    
    	switch (currentCol.inputItemProperty) {
			case "contents" :
				if(!(fileContents))
				{
					//TODO load file contents
					fileContents= item.name_stem;
				}
				fileSubject = fileContents;
				break;
			case "ext" :
				fileSubject = item.ext;
				break;
			case "name" :
				fileSubject = item.name;
				break;
			case "realpath" :
				fileSubject = item.realpath;
				break;
			case "name_stem" :
			default:
				//fileSubject = item[columnConfigurations[i].inputItemProperty];;
				fileSubject = item.name_stem;
				break;
		} 

	    // Add prefix to the subject if present
    	subject = (typeof currentCol.subjectPrefix === 'undefined') ? fileSubject : currentCol.subjectPrefix + fileSubject;   
    	//result = input.match( new RegExp(RegExColumns[scriptColData.col].regexp));
    	regexMatch = (new RegExp(currentCol.regexp)).exec(subject);

	    if (regexMatch != null)    {
	      	if (currentCol.firstValid) {
	         	for(i = 1; i < regexMatch.length + 1; i++) {
	            	if (regexMatch[i] != '') {
	               		result[currentCol.name] = regexMatch[i];
	               		break;
	           	 	}	
	         	}
	      	}
	      	else {
	         	var capturegroup = (typeof currentCol.group === 'undefined') ? 0 : currentCol.group;
	         	result[currentCol.name] = regexMatch[capturegroup];
	      	}
	    }
	}

	return result;
}

function parseDate(input, format) {
	if(!input) return;

	var parsedDate = new Date(Date.parse(input));
	if(!parsedDate || parsedDate == "NaN")
    {

		format = format || 'yyyy-mm-dd'; // default format
		var parts = input.match(/(\d+)/g), 
		  i = 0, fmt = {};
	  	if(parts)
		{
			// extract date-part indexes from the format
			format.replace(/(yyyy|dd|mm)/g, function(part) { fmt[part] = i++; });
			parsedDate = new Date(parts[fmt['yyyy']], parts[fmt['mm']]-1, parts[fmt['dd']]);	
		}
    }

	return (!parsedDate || parsedDate == "NaN") ? null : parsedDate;
}

function InArray(col,arr){
	for (var i=0;i<arr.length;i++){
	if (col==arr[i].name) return true;
	}
}

==SCRIPT RESOURCES
<resources>
	<resource name="RegExpColumnsConfigure" type="dialog">
		<dialog fontsize="8" height="312" lang="english" resize="yes" title="Regexp custom columns" width="318">
			<control close="0" height="14" name="buttonClose" resize="xy" title="Close" type="button" width="50" x="252" y="288" />
			<control height="300" name="columnsGroup" resize="h" title="RegExp Columns" type="group" width="96" x="6" y="6" />
			<control height="246" name="columnsList" resize="h" type="listbox" width="84" x="12" y="18" />
			<control halign="left" height="8" name="columnNameSatic" title="Name" type="static" width="66" x="108" y="24" />
			<control halign="left" height="12" name="columnName" resize="w" tip="Not set" type="edit" width="132" x="174" y="24" />
			<control halign="left" height="8" name="columnDisplayNameStatic" title="Display Name" type="static" width="66" x="108" y="42" />
			<control halign="left" height="12" name="columnDisplayName" resize="w" tip="Not set (defaults to Name)" type="edit" width="132" x="174" y="42" />
			<control halign="left" height="8" name="columnTypeStatic" title="Column Type" type="static" width="66" x="108" y="60" />
			<control halign="left" height="8" name="columnDefaultWidthStatic" title="Default Width" type="static" width="66" x="108" y="114" />
			<control halign="left" height="12" name="columnDefaultWidth" number="yes" tip="Not set" title="80" type="edit" width="66" x="174" y="114" />
			<control height="40" name="columnType" type="combo" width="102" x="174" y="60">
				<contents>
					<item text="text" />
					<item text="number" />
					<item text="double" />
					<item text="size" />
					<item text="zip" />
					<item text="percent" />
					<item text="graph" />
					<item text="igraph" />
					<item text="date" />
					<item text="time" />
					<item text="datetime" />
					<item text="stars" />
				</contents>
			</control>
			<control header="yes" height="64" name="group1" resize="yw" type="group" width="210" x="102" y="270" />
			<control default="yes" height="14" name="buttonApply" resize="xy" title="Appy Changes" type="button" width="74" x="234" y="252" />
			<control height="14" name="buttonAdd" resize="y" title="Add Column" type="button" width="60" x="24" y="270" />
			<control height="14" name="buttonDelete" resize="y" title="Delete Column" type="button" width="60" x="24" y="288" />
			<control height="14" name="buttonClipboard" resize="y" title="Copy settings to clipboard" type="button" width="102" x="114" y="288" />
			<control header="yes" height="64" name="groupregexp" resize="w" title="Regexp Settings" type="group" width="204" x="108" y="162" />
			<control halign="left" height="8" name="columnRegExpStatic" title="RegExp" type="static" width="66" x="108" y="174" />
			<control halign="left" height="12" name="columnRegExp" resize="w" tip="Not set I.E. (.*)" type="edit" width="132" x="174" y="174" />
			<control header="yes" height="64" name="groupColumnSettings" resize="w" title="Column Settings" type="group" width="204" x="108" y="12" />
			<control height="10" name="columnEnableGrouping" title="Enable Grouping" type="check" width="78" x="174" y="132" />
			<control height="10" name="columnEnableSort" title="Enable Sorting" type="check" width="84" x="174" y="150" />
			<control halign="left" height="8" name="columnJustifyStatic" title="Column Justify" type="static" width="66" x="108" y="96" />
			<control height="40" name="columnJustify" type="combo" width="102" x="174" y="96">
				<contents>
					<item text="left" />
					<item text="right" />
					<item text="center " />
					<item text="path" />
				</contents>
			</control>
			<control halign="left" height="8" name="columnDateTimeFormatStatic" title="Date Time Format" type="static" width="60" x="108" y="78" />
			<control enable="no" halign="left" height="12" name="columnDateTimeFormat" tip="yyyy-MM-dd" type="edit" width="102" x="174" y="78" />
			<control halign="left" height="8" name="columnCaptureGroupStatic" title="Capture Group" type="static" width="60" x="108" y="192" />
			<control halign="left" height="12" name="columnCaptureGroup" number="yes" type="edit" width="64" x="174" y="192" />
			<control halign="left" height="8" name="columnInputStatic" title="Input" type="static" width="66" x="108" y="210" />
			<control height="40" name="columnInput" type="combo" width="102" x="174" y="210">
				<contents>
					<item text="name_stem" />
					<item text="realpath" />
					<item text="name" />
					<item text="ext" />
				</contents>
			</control>
			<control halign="left" height="8" name="columnFilterStatic" title="Filter" type="static" width="20" x="108" y="234" />
			<control height="40" name="columnFilter" type="combo" width="102" x="174" y="228">
				<contents>
					<item text="All" />
					<item text="Files" />
					<item text="Folders" />
				</contents>
			</control>
			<control height="12" name="columnFirstValid" title="Take First good one" type="check" width="64" x="240" y="192" />
		</dialog>
	</resource>
</resources>
